/*
 * kdr-say.c - Kedrigern (Milena TTS server for Raspberry Pi)
 * Copyright (C) Bohdan R. Rau 2019 <ethanak@polip.com>
 *
 * Kedrigern is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * Kedrigern is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Kedrigern.  If not, write to:
 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>

#include <kedrigern.h>

struct kdr_Voice k;

void helpme(int ex)
{
    fprintf(ex ? stderr : stdout,"Sposób użycia:\n\
  kdr-stop [-v plik_głosu] [-P port] [-H host_ip]\n\
  kdr-speaking [-v plik_głosu] [-P port] [-H host_ip]\n\
  kdr-say [-v plik_głosu] [-P port] [-H host_ip] [opcje] [tekst do powiedzenia]\n\
gdzie opcje to:\n\
  -t <float> - tempo (0.5 .. 2.0)\n\
  -f <freq>  - częstotliwość bazowa (0.5 .. 2.0)\n\
  -l <freq>  - częstotliwość próbkowania głosu (8000 .. 32000)\n\
  -r <freq>  - częstotliwość odtwarzania (8000 .. 32000)\n");
    exit(ex);
}

int good_int(const char *arg, int mini, int maxi)
{
    char *c;
    int val;
    val = strtol(arg, &c, 10);
    if (*c || val < mini || val > maxi) return -1;
    return val;
}

double good_float(const char *arg)
{
    char *c;
    double val;
    val = strtod(arg, &c);
    if (*c || val < 0.5 || val > 2.0) return -1.0;
    return val;
}

int good_ip(char *arg)
{
    char *c, *d;
    int i,n;
    for (i=0; i< 4; i++) {
        if (i && *arg++ != '.') return 0;
        if (*arg == '0' && isdigit(arg[1])) return 0;
        if (!isdigit(*arg)) return 0;
        n=strtol(arg, &arg, 10);
        if (n < 0 || n > 255) return 0;
    }
    if (*arg) return 0;
    return 1;
}

void readVoiceFile(const char *fname)
{
    FILE *f=fopen(fname,"r");
    char buf[1024];
    int i;
    static const char *cmds[]={"host","port",
            "tempo","freq","samplefreq","samplerate",NULL};
    if (!f && *fname != '/') {
        strcpy(buf,KEDRIGERNPATH);
        strcat(buf,"/");
        strcat(buf,fname);
        f=fopen(buf,"r");
        if (!f) {
            strcat(buf,".voice");
            f=fopen(buf,"r");
        }
    }
    if (!f) {
        perror(fname);
        exit(1);
    }
    while(fgets(buf, 1024, f)) {
        char *c=buf, *d;int n; double t;
        while (*c && isspace(*c)) c++;
        d=strchr(c,'#');
        if (d) d--;
        else d = c+strlen(c)-1;
        while (d >= c) {
            if (!isspace(*d)) break;
            d--;
        }
        d[1]=0;
            
        if (!*c || *c == '#') continue;
        for (i=0; cmds[i]; i++) {
            if (strncasecmp(c, cmds[i], strlen(cmds[i]))) continue;
            c += strlen(cmds[i]);
            while (*c && isspace(*c)) c++;
            if (*c++!='=') goto badformat;
            while (*c && isspace(*c)) c++;
            break;
        }
        if (!cmds[i]) goto badformat;
        switch(i) {
            case 0: // host
            if (!strcmp(c, "localhost")) strcpy(k.ip,"127.0.0.1");
            else if (good_ip(c)) strcpy(k.ip, c);
            else goto badformat;
            break;
            
            case 1: // port
            n=good_int(c,1,65535);
            if (n < 0) goto badformat;
            k.port = n;
            break;
            
            case 2: // tempo
            t = good_float(c);
            if (t < 0) goto badformat;
            k.tempo = t;
            break;
            
            case 3: // freq
            t = good_float(c);
            if (t < 0) goto badformat;
            k.freq = t;
            break;
            
            case 4: //samplefreq
            n=good_int(c, 8000, 32000);
            if (n < 0) goto badformat;
            k.samplefreq = n;
            break;
            
            case 5: //samplerate
            n=good_int(c, 8000, 32000);
            if (n < 0) goto badformat;
            k.samplerate = n;
            break;
        }
    }  
    fclose(f);
    return;
badformat:
    fprintf(stderr, "Błędna linia w pliku głosu: %s\n", buf);
    exit(1);
}

int main(int argc, char *argv[])
{
    int opt, n, v, is_say,is_stop=0,is_speaking=0;
    double f;

    kdr_InitVoice(&k);
    char *c=strrchr(argv[0],'/');
    if (c) c++;
    else c=argv[0];
    is_say = !strcmp(c, "kdr-say");
    is_stop = !strcmp(c, "kdr-stop");
    is_speaking = !strcmp(c, "kdr-speaking");
    if (!is_say && !is_stop && !is_speaking) helpme(1);
    
    while ((opt = getopt(argc, argv, "hH:P:r:l:f:t:v:")) != -1) {
        switch(opt) {
            case 'v':
            readVoiceFile(optarg);
            break;
            
            case 'H':
            if (!strcmp(optarg,"localhost")) strcpy(k.ip,"127.0.0.1");
            if (!good_ip(optarg)) helpme(1);
            strcpy(k.ip, optarg);
            break;
            
            case 'P':
            if ((n = good_int(optarg, 1, 65535)) < 0) helpme(1);
            k.port = n;
            break;
            
            case 'l':
            if (!is_say || (n = good_int(optarg, 8000, 32000)) < 0) helpme(1);
            k.samplefreq = n;
            break;

            case 'r':
            if (!is_say || (n = good_int(optarg, 8000, 32000)) < 0) helpme(1);
            k.samplerate = n;
            break;
            
            case 't':
            if (!is_say || (f = good_float(optarg)) < 0) helpme(1);
            k.tempo = f;
            break;
            
            case 'f':
            if (!is_say || (f = good_float(optarg)) < 0) helpme(1);
            k.freq = f;
            break;
            
            case 'h':
            helpme(0);

            default:
            helpme(1);
        }
    }
    if (optind < argc && !is_say) helpme(1);
    if (is_stop) {
        kdr_Stop(&k);
        exit(0);
    }
    if (is_speaking) {
        n = kdr_IsSpeaking(&k);
        if (n < 0) n = 2;
        exit(n);
    }
    char *str;
    if (optind >= argc) {
        char buf[1024];
        fgets(buf, 1024, stdin);
        str=buf;
    }
    else {
        int i,len;
        for (i = optind, len = 0; i < argc; i++) {
            len += strlen(argv[i]) + 1;
        }
        str = malloc(len);
        for (i = optind; i < argc; i++) {
            if (i == optind) strcpy(str,argv[i]);
            else {
                strcat(str," ");
                strcat(str,argv[i]);
            }
        }
    }
    kdr_Say(str,&k);
    exit(0);
        
}
